/* Crista/SPAS TANS Recorder Packet viewer */
/* NOTE: This code assumes Intel byte ordering (LSB first) */
/*       The TANS recorder data is (as on packet description) MSB first */


#include "stdio.h"
#include "stdlib.h"

unsigned short bufpc[200];
unsigned short buf[200];

union u {
	double d;
	unsigned short i[4];
} convl2d;

union u1 {
	float f;
	unsigned short i[2];
} convi2f;

long templ;

struct TagRaw {
	char	sv;
	float	codephase;
	float	doppler;
} raw[6];
	

void ReadE4(short i);

	
void main(short argc, char *argv[])
{
	FILE *f;
        FILE *fjack;
	char fname[30], ch, ec, ecode, scode;
	short t_inc = 0, err, i, fix, stop = 0, searching = 0, start = 0;

	float pos[4], vel[4], att[3], t_pf, t_af, dop[4];
	float zero = 0.0;
	char best4svs[4];
		
        if (argc == 1) {
                printf("\nEnter input filename: ");
                scanf("%s", fname);
        }
        else if (argc == 2) {
                strcpy(fname, argv[1]);
        }
        else return;

	f = fopen(fname, "rb");
	if (f == NULL)	{ printf("\nError opening file\n"); exit(1); }

	fjack = fopen("jackview.dat", "w+");
	if (fjack == NULL)	{ printf("\nError opening file ""jackview.dat"" \n"); exit(1); }


	while (!stop)
	{
		do {
			if (fread(&bufpc[0], 2, 32, f) == 0) stop = 1;
			else {
				for (i = 0; i < 32; i++)
					buf[i] = ((bufpc[i] << 8) & 0xff00) | ((bufpc[i] >> 8) & 0xff);

				stop = 0;
			}

		} while ((buf[0] != 0x0203) && (!stop)) ;

		if (stop) break;
		if (buf[3] == 1) start = 1;

		if (start)
		{
			switch (buf[3])
			{
				case 1:
					ReadE4(0);
					convi2f.i[0] = buf[22]; convi2f.i[1] = buf[23]; att[0] = convi2f.f;
					convi2f.i[0] = buf[24]; convi2f.i[1] = buf[25]; att[1] = convi2f.f;
					convi2f.i[0] = buf[26]; convi2f.i[1] = buf[27]; att[2] = convi2f.f;
					break;
	
				case 2:
					ReadE4(1);
					convi2f.i[0] = buf[22]; convi2f.i[1] = buf[23]; pos[0] = convi2f.f;
					convi2f.i[0] = buf[24]; convi2f.i[1] = buf[25]; pos[1] = convi2f.f;
					convi2f.i[0] = buf[26]; convi2f.i[1] = buf[27]; pos[2] = convi2f.f;
					convi2f.i[0] = buf[30]; convi2f.i[1] = buf[31]; pos[3] = convi2f.f;
			
					convi2f.i[0] = buf[28]; convi2f.i[1] = buf[29]; t_pf = convi2f.f;
					break;
	
				case 3:
					ReadE4(2);
					convi2f.i[0] = buf[22]; convi2f.i[1] = buf[23]; vel[0] = convi2f.f;
					convi2f.i[0] = buf[24]; convi2f.i[1] = buf[25]; vel[1] = convi2f.f;
					convi2f.i[0] = buf[26]; convi2f.i[1] = buf[27]; vel[2] = convi2f.f;
					convi2f.i[0] = buf[30]; convi2f.i[1] = buf[31]; vel[3] = convi2f.f;
					break;
	
				case 4:
					ReadE4(3);
					best4svs[0] = buf[22] >> 8;
					best4svs[1] = buf[22] & 0xff;
					best4svs[2] = buf[23] >> 8;
					best4svs[3] = buf[23] & 0xff;

					convi2f.i[0] = buf[24]; convi2f.i[1] = buf[25]; dop[0] = convi2f.f;	
					convi2f.i[0] = buf[26]; convi2f.i[1] = buf[27]; dop[1] = convi2f.f;	
					convi2f.i[0] = buf[28]; convi2f.i[1] = buf[29]; dop[2] = convi2f.f;	
					convi2f.i[0] = buf[30]; convi2f.i[1] = buf[31]; dop[3] = convi2f.f;	
					break;
	
				case 5:
					ReadE4(4);
					scode = buf[22] >> 8;
					ecode = buf[22] & 0xff;
					break;
	
				case 6:
					ReadE4(5);
					fprintf(fjack,"\n%6.3f", t_pf);
					fprintf(fjack," %10.1f %10.1f %10.1f %10.1f", pos[0], pos[1], pos[2], pos[3]);
					fprintf(fjack," %7.2f %7.2f %7.2f %7.1f", vel[0], vel[1], vel[2], vel[3]);
					fprintf(fjack," %9.3f %9.3f %9.3f", att[0], att[1], att[2]);
					fprintf(fjack," %9.3f %9.3f %9.3f", zero, zero, zero);
					fprintf(fjack," %9.3f %9.3f", zero, zero);

					for (i = 0; i < 6; i++) {
						fprintf(fjack," %3d %8.1f %8.1f", raw[i].sv, raw[i].codephase, raw[i].doppler);
					}
					break;
				
				default:
					printf("\n Not a TANS Packet");
					break;
			}
		 
		}
	}	
	printf("\n");
        fclose(fjack);
	fclose(f);
}


void ReadE4(short i)
{
	raw[i].sv = buf[4] & 0xff;
	convi2f.i[0] = buf[7]; convi2f.i[1] = buf[8];
	raw[i].codephase = convi2f.f;
	convi2f.i[0] = buf[9]; convi2f.i[1] = buf[10];
	raw[i].doppler = convi2f.f;
}

