/****************************************************************************
 * CRISTA-SPAS GPS state vector packet viewer (id 0x04ff)
 *   Displays in ASCII:
 *         Position X,Y,Z, Velocity X,Y,Z, correlated time, datation time
 *
 */


#include "math.h"
#include "stdio.h"
#include "stdlib.h"
#define TWO_TO_39 549755813888.0

double Conv48toDouble(unsigned short *bp);
unsigned short buf[200], bufpc[200];

void main(int argc, char *argv[])
{
	FILE *f;
        FILE *fal;
	char fname[30];
	int i, stop = 0;
	unsigned short sec, milli, hours, minutes, days;	
	double pos[3], vel[3], time;
        long msm, lsm, expl;
        double mant, frac, expf;


		
        if (argc == 1) {
                printf("\nEnter input filename: ");
                scanf("%s", fname);
        }
        else if (argc == 2) {
                strcpy(fname, argv[1]);
        }
        else return;

	f = fopen(fname, "rb");
	if (f == NULL)	{ printf("\nError opening file\n"); return; }

	fal = fopen("alcatel.dat", "w+");
	if (fal == NULL)	{ printf("\nError opening file\n"); return; }

	printf("Processing file %s... ",fname);
        fflush(stdout);

	while (!stop)
	{
		do {
			if (fread(&bufpc[0], 2, 32, f) == 0) stop = 1;
			else {
				for (i = 0; i < 32; i++)
					buf[i] = ((bufpc[i] << 8) & 0xff00) | ((bufpc[i] >> 8) & 0xff);
				stop = 0;
			}

		} while ((buf[0] != 0x04FF) && (!stop)) ;

		if (stop) break;


        msm = (buf[4] << 16) | (buf[5] & 0xFF00);
        lsm = buf[6];
        expl = buf[5] & 0x00FF;
        mant = msm;
        mant = (mant*256) + (double)lsm;
        frac = mant /TWO_TO_39;
        expf = (signed char) expl;
        pos[0] = frac*pow(2.0,expf); 


        msm = (buf[7] << 16) | (buf[8] & 0xFF00);
        lsm = buf[9];
        expl = buf[8] & 0x00FF;
        mant = msm;
        mant = (mant*256) + (double)lsm;
        frac = mant /TWO_TO_39;
        expf = (signed char) expl;
        pos[1] = frac*pow(2.0,expf); 

        msm = (buf[10] << 16) | (buf[11] & 0xFF00);
        lsm = buf[12];
        expl = buf[11] & 0x00FF;
        mant = msm;
        mant = (mant*256) + (double)lsm;
        frac = mant /TWO_TO_39;
        expf = (signed char) expl;
        pos[2] = frac*pow(2.0,expf); 

        msm = (buf[13] << 16) | (buf[14] & 0xFF00);
        lsm = buf[15];
        expl = buf[14] & 0x00FF;
        mant = msm;
        mant = (mant*256) + (double)lsm;
        frac = mant /TWO_TO_39;
        expf = (signed char) expl;
        vel[0] = frac*pow(2.0,expf); 

        msm = (buf[16] << 16) | (buf[17] & 0xFF00);
        lsm = buf[18];
        expl = buf[17] & 0x00FF;
        mant = msm;
        mant = (mant*256) + (double)lsm;
        frac = mant /TWO_TO_39;
        expf = (signed char) expl;
        vel[1] = frac*pow(2.0,expf); 

        msm = (buf[19] << 16) | (buf[20] & 0xFF00);
        lsm = buf[21];
        expl = buf[20] & 0x00FF;
        mant = msm;
        mant = (mant*256) + (double)lsm;
        frac = mant /TWO_TO_39;
        expf = (signed char) expl;
        vel[2] = frac*pow(2.0,expf); 

		sec = buf[22] >> 10;
		milli = buf[22] & 0x03ff;
		hours = buf[23] >> 8;
		minutes = buf[23] & 0x003f;
		days = buf[24];
		time = (hours * 3600.0) + (minutes * 60.0) + (sec * 1.0) + (milli / 1000.0);

		fprintf(fal," %10.1f %10.1f %10.1f ", pos[0], pos[1], pos[2]);
		fprintf(fal," %9.2f %9.2f %9.2f ", vel[0], vel[1], vel[2]);
/*		printf(" %4i %10.3f ", days, time);
*/
                fprintf(fal, " %10.3f \n",time);
/*
		sec = buf[25] >> 10;
		milli = buf[25] & 0x03ff;
		hours = buf[26] >> 8;
		minutes = buf[26] & 0x003f;
		days = buf[27];
		time = (hours * 3600.0) + (minutes * 60.0) + (sec * 1.0) + (milli / 1000.0);
		fprintf(fal," %4i %10.3f\n ", days, time);
*/
	}	
	printf("Complete. \n");
        fclose(fal);
	fclose(f);
        return;
}


double Conv48toDouble(unsigned short *bp)
{
        long msm, lsm, expl;
        double mant, frac, expf, answer;

	msm = *((long *)bp) & 0xffffff00; 
	lsm = bp[2];
	expl = bp[1] & 0x000000ff;

        mant = msm;
        mant = (mant * 256.0) + (double)lsm;
        frac = mant / TWO_TO_39;
        expf = (signed char) expl;
        answer = frac * pow(2.0, expf);
	return(answer);
}
